#!/usr/bin/env python

#
# Lexer.py
#
# Lexer using ply.lex
# Converts the input file into a collection of token, so be parsed
#
#
# Copyright (C) 2007  Sylvain Munaut <tnt@246tNt.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import ply.lex as lex


tokens = (
	'LABEL', 'LABEL_LOCAL', 'DIRECTIVE', 'INSTRUCTION',
	'IDENTIFIER', 'IDENTIFIER_LOCAL', 'NUMBER', 'QSTRING', 'COMMA',
	'LPAREN', 'RPAREN', 'PLUS', 'MINUS', 'TIMES', 'DIVIDES',
	'NEWLINE',
)


class Lexer:

	def __init__(self):
		self.tokens = tokens

	states = (
		('main', 'exclusive'),
		('args', 'exclusive'),
	)



	def t_main_LABEL(self, t):
		r'[a-zA-Z_.$][a-zA-Z0-9_.$]*:'
		t.value = t.value[0:len(t.value)-1]
		return t

	def t_main_LABEL_LOCAL(self, t):
		r'[0-9]+:'
		t.value = t.value[0]
		return t

	def t_main_DIRECTIVE(self, t):
		r'\.[a-zA-Z_.$][a-zA-Z0-9_.$]*'
		t.value = t.value[1:len(t.value)]
		t.lexer.begin('args')
		return t

	def t_main_INSTRUCTION(self, t):
		r'[a-zA-Z_.$][a-zA-Z0-9_.$]*'
		t.lexer.begin('args')
		return t

	def t_main_newline(self, t):
		r'\n+'
		t.lexer.lineno += t.value.count("\n")


	t_args_IDENTIFIER	= r'[a-zA-Z_.$][a-zA-Z0-9_.$]*'
	t_args_COMMA		= r','
	t_args_LPAREN		= r'\('
	t_args_RPAREN		= r'\)'
	t_args_PLUS		= r'\+'
	t_args_MINUS		= r'-'
	t_args_TIMES		= r'\*'
	t_args_DIVIDES		= r'/'

	def t_args_IDENTIFIER_LOCAL(self, t):
		r'[0-9][fb]'
		return t

	def t_args_NUMBER(self, t):
		r'(0x[0-9a-fA-F]+)|([1-9][0-9]*)|(0[0-9]*)'
		try:
			t.value = int(eval(t.value));
		except ValueError:
			print "Integer value too large", t.value
			t.value = 0
		return t

	def t_args_QSTRING(self, t):
		r'\"[^"]*\"'
		t.value = eval(t.value)
		return t


	def t_args_NEWLINE(self, t):
		r'\n+'
		t.lexer.lineno += t.value.count("\n")
		t.lexer.begin('main')
		return t


	def t_ANY_comment(self, t):
		# Supports /* */  and ;
		r'(/\*(.|\n)*?\*/)|(;.*)'
		pass



	t_ANY_ignore	= " \t\r"

	def t_ANY_error(self, t):
		print "Illegal character '%s'" % t.value[0]
		t.lexer.skip(1)



	def build(self, **kwargs):
		self.lexer = lex.lex(object=self, **kwargs)
		self.lexer.begin('main')

